@@ -1,26 +1,55 @@
 import hashlib
 from itertools import zip_longest
 
-from ccinput.exceptions import InvalidParameter, InternalError, ImpossibleCalculation, \
-                               MissingParameter
-from ccinput.utilities import get_abs_software, get_method, get_abs_basis_set, \
-                              get_abs_solvent, get_theory_level, standardize_memory, \
-                              get_npxyz, get_coord, has_dispersion_parameters, warn
+from ccinput.exceptions import (
+    InvalidParameter,
+    InternalError,
+    ImpossibleCalculation,
+    MissingParameter,
+)
+from ccinput.utilities import (
+    get_abs_software,
+    get_method,
+    get_abs_basis_set,
+    get_abs_solvent,
+    get_theory_level,
+    standardize_memory,
+    get_npxyz,
+    get_coord,
+    has_dispersion_parameters,
+    warn,
+)
 from ccinput.constants import ATOMIC_NUMBER, SYN_SOFTWARE
 
+
 class Calculation:
     """
-        Holds all the data required to generate an input file.
-        Its fields are the parameters likely to change (charge, multiplicity,
-        xyz, calculation type...). The other parameters are contained in the
-        Parameters class (accessed through self.parameters).
+    Holds all the data required to generate an input file.
+    Its fields are the parameters likely to change (charge, multiplicity,
+    xyz, calculation type...). The other parameters are contained in the
+    Parameters class (accessed through self.parameters).
     """
-    def __init__(self, xyz, parameters, type, constraints=[], nproc=0, mem=0,
-            charge=0, multiplicity=1, aux_name="calc2", name="calc",
-            header="File created by ccinput", software=""):
+
+    def __init__(
+        self,
+        xyz,
+        parameters,
+        type,
+        constraints=[],
+        nproc=1,
+        mem=1000,
+        charge=0,
+        multiplicity=1,
+        aux_name="calc2",
+        name="calc",
+        header="File created by ccinput",
+        software="",
+        file=None,
+    ):
         self.xyz = xyz
         self.parameters = parameters
         self.type = type
+        self.file = file
 
         if software not in SYN_SOFTWARE:
             raise InvalidParameter(f"Invalid software: '{software}'")
@@ -34,10 +63,14 @@ class Calculation:
             raise InvalidParameter(f"Invalid number of cores: '{nproc}'")
 
         if abs(self.nproc - float(nproc)) > 1e-4:
-            raise InvalidParameter(f"The number of cores must be an integer (received '{nproc}')")
+            raise InvalidParameter(
+                f"The number of cores must be an integer (received '{nproc}')"
+            )
 
         if self.nproc < 1:
-            raise InvalidParameter(f"The number of cores must at least 1 (received '{nproc}')")
+            raise InvalidParameter(
+                f"The number of cores must at least 1 (received '{nproc}')"
+            )
 
         try:
             self.mem = standardize_memory(mem)
@@ -59,9 +92,13 @@ class Calculation:
             raise InvalidParameter(f"Invalid multiplicity: '{multiplicity}'")
 
         if abs(self.multiplicity - float(multiplicity)) > 1e-4:
-            raise InvalidParameter(f"Multiplicity must be an integer (received '{multiplicity}')")
+            raise InvalidParameter(
+                f"Multiplicity must be an integer (received '{multiplicity}')"
+            )
         if self.multiplicity < 1:
-            raise InvalidParameter(f"Multiplicity must at least 1 (received '{multiplicity}')")
+            raise InvalidParameter(
+                f"Multiplicity must at least 1 (received '{multiplicity}')"
+            )
         self.verify_charge_mult()
         self.constraints = constraints
 
@@ -70,10 +107,10 @@ class Calculation:
         self.header = header
 
     def verify_charge_mult(self):
-        """ Verifies that the requested charge and multiplicity are possible for the structure """
+        """Verifies that the requested charge and multiplicity are possible for the structure"""
         electrons = 0
-        for line in self.xyz.split('\n'):
-            if line.strip() == '':
+        for line in self.xyz.split("\n"):
+            if line.strip() == "":
                 continue
             el = line.split()[0]
             electrons += ATOMIC_NUMBER[el]
@@ -83,19 +120,35 @@ class Calculation:
         odd_m = self.multiplicity % 2
 
         if odd_e == odd_m:
-            raise ImpossibleCalculation(f"This combination of charge ({self.charge}) " +
-                    f"and multiplicity ({self.multiplicity}) is impossible")
+            raise ImpossibleCalculation(
+                f"This combination of charge ({self.charge}) "
+                + f"and multiplicity ({self.multiplicity}) is impossible"
+            )
+
 
 class Parameters:
     """
-        Holds all the parameters about the computational method.
-        These parameters do not depend on the particular system
-        (e.g. regarding the charge and multiplicity) and can be reused.
+    Holds all the parameters about the computational method.
+    These parameters do not depend on the particular system
+    (e.g. regarding the charge and multiplicity) and can be reused.
     """
 
-    def __init__(self, software, solvent="", solvation_model="", solvation_radii="",
-            custom_solvation_radii="", basis_set="", method="", specifications="",
-            density_fitting="", custom_basis_sets="", d3=False, d3bj=False, **kwargs):
+    def __init__(
+        self,
+        software,
+        solvent="",
+        solvation_model="",
+        solvation_radii="",
+        custom_solvation_radii="",
+        basis_set="",
+        method="",
+        specifications="",
+        density_fitting="",
+        custom_basis_sets="",
+        d3=False,
+        d3bj=False,
+        **kwargs,
+    ):
 
         self.solvent = get_abs_solvent(solvent)
         if self.solvent != "":
@@ -104,8 +157,9 @@ class Parameters:
             self.custom_solvation_radii = custom_solvation_radii.lower()
 
             if self.solvation_model.strip() == "":
-                raise InvalidParameter("No solvation model specified, " +
-                            "although solvation is requested")
+                raise InvalidParameter(
+                    "No solvation model specified, " + "although solvation is requested"
+                )
             if self.solvation_radii.strip() == "":
                 warn("No solvation radii specified; using default radii")
         else:
@@ -116,33 +170,42 @@ class Parameters:
         self.software = get_abs_software(software)
 
         if method == "":
-            if 'functional' in kwargs:
-                method = get_method(kwargs['functional'], self.software)
+            if "functional" in kwargs:
+                method = get_method(kwargs["functional"], self.software)
+            elif self.software == "xtb":
+                method = "GFN2-xTB"
             else:
                 raise InvalidParameter("No calculation method specified (method='...')")
+
         else:
             self.method = get_method(method, self.software)
 
         self.theory_level = get_theory_level(method)
 
-        if self.theory_level not in ["semi-empirical", "xtb", "special"]:
+        if self.theory_level not in ["semiempirical", "xtb", "special"]:
             self.basis_set = get_abs_basis_set(basis_set)
         else:
             self.basis_set = ""
 
         if d3 and d3bj:
-            raise InvalidParameter("Cannot use both D3(0) and D3BJ dispersion corrections")
+            raise InvalidParameter(
+                "Cannot use both D3(0) and D3BJ dispersion corrections"
+            )
 
         self.d3 = d3
         self.d3bj = d3bj
 
-        if d3 and not has_dispersion_parameters(self.method, version='d3'):
-            warn(f"Your calculation requests a method ({self.method}) that may not have D3 "
-                 "parameters. Be aware that the calculation might result in an error")
+        if d3 and not has_dispersion_parameters(self.method, version="d3"):
+            warn(
+                f"Your calculation requests a method ({self.method}) that may not have D3 "
+                "parameters. Be aware that the calculation might result in an error"
+            )
 
-        if d3bj and not has_dispersion_parameters(self.method, version='d3bj'):
-            warn(f"Your calculation requests a method ({self.method}) that may not have D3BJ "
-                 "parameters. Be aware that the calculation might result in an error")
+        if d3bj and not has_dispersion_parameters(self.method, version="d3bj"):
+            warn(
+                f"Your calculation requests a method ({self.method}) that may not have D3BJ "
+                "parameters. Be aware that the calculation might result in an error"
+            )
 
         self.specifications = specifications
         self.density_fitting = density_fitting
@@ -150,18 +213,18 @@ class Parameters:
         self.kwargs = kwargs
 
     def __eq__(self, other):
-        values = [(k,v) for k,v in self.__dict__.items()]
-        other_values = [(k,v) for k,v in other.__dict__.items()]
+        values = [(k, v) for k, v in self.__dict__.items()]
+        other_values = [(k, v) for k, v in other.__dict__.items()]
 
         return values == other_values
 
     @property
     def md5(self):
         """
-            Returns a hash digest of the parameters to easily compare
-            parameters without using objects.
+        Returns a hash digest of the parameters to easily compare
+        parameters without using objects.
         """
-        values = [(k,v) for k,v in self.__dict__.items()]
+        values = [(k, v) for k, v in self.__dict__.items()]
         params_str = ""
         for k, v in values:
             if isinstance(v, int):
@@ -170,26 +233,41 @@ class Parameters:
                 params_str += f"{k}={v.lower()};"
             else:
                 raise InternalError("Unknown value type")
-        hash = hashlib.md5(bytes(params_str, 'UTF-8'))
+        hash = hashlib.md5(bytes(params_str, "UTF-8"))
         return hash.digest()
 
+
 class Constraint:
     """
-        Class to contain a single constraint (freeze, scan and the like) and provide useful functions for it.
+    Class to contain a single constraint (freeze, scan and the like) and provide useful functions for it.
     """
-    def __init__(self, scan=False, start_d=None, end_d=None, step_size=None,
-                 num_steps=None, ids=[], xyz=None, software=""):
+
+    def __init__(
+        self,
+        scan=False,
+        start_d=None,
+        end_d=None,
+        step_size=None,
+        num_steps=None,
+        ids=[],
+        xyz=None,
+        software="",
+    ):
         self.scan = scan
-        self.ids = ids # One-indexed (Gaussian like, not like ORCA)
+        self.ids = ids  # One-indexed (Gaussian like, not like ORCA)
         self.software = software
 
         if len(ids) < 2 or len(ids) > 4:
-            raise InvalidParameter(f"Invalid number of atoms: {len(ids)}, needs to be between 2 and 4")
+            raise InvalidParameter(
+                f"Invalid number of atoms: {len(ids)}, needs to be between 2 and 4"
+            )
 
         _xyz = get_npxyz(xyz)
 
         if max(ids) > len(_xyz):
-            raise InvalidParameter(f"Invalid atom index {max(ids)}: larger than the number of atoms")
+            raise InvalidParameter(
+                f"Invalid atom index {max(ids)}: larger than the number of atoms"
+            )
         if len(ids) != len(set(ids)):
             raise InvalidParameter(f"The provided atom indices are not all unique")
 
@@ -202,17 +280,23 @@ class Constraint:
             if num_params < 2:
                 raise InvalidParameter("Not enough constraint parameters given")
             elif num_params == 3:
-                raise InvalidParameter("Too many constraint parameters given, more than one possibility of unique parameters")
+                raise InvalidParameter(
+                    "Too many constraint parameters given, more than one possibility of unique parameters"
+                )
 
             if start_d is not None and end_d is not None and xyz is None:
-                raise InvalidParameter("The XYZ structure needs to be specified in order to calculate the initial coordinate value")
+                raise InvalidParameter(
+                    "The XYZ structure needs to be specified in order to calculate the initial coordinate value"
+                )
 
-        if software != 'gaussian':
+        if software != "gaussian":
             self.start_d = start_d
         else:
             if start_d is not None:
-                warn("Gaussian only allows scans that start from the current structure; " +
-                     "overriding the specified initial value")
+                warn(
+                    "Gaussian only allows scans that start from the current structure; "
+                    + "overriding the specified initial value"
+                )
             self.start_d = None
 
         self.end_d = end_d
@@ -230,13 +314,13 @@ class Constraint:
             assert self.step_size is not None
             assert self.num_steps is not None
 
-            self.end_d = self.start_d + self.num_steps*self.step_size
+            self.end_d = self.start_d + self.num_steps * self.step_size
 
         if self.step_size is None:
             assert self.end_d is not None
             assert self.num_steps is not None
 
-            self.step_size = round((self.end_d-self.start_d)/self.num_steps, 2)
+            self.step_size = round((self.end_d - self.start_d) / self.num_steps, 2)
 
         if self.num_steps is None:
             assert self.end_d is not None
@@ -245,13 +329,13 @@ class Constraint:
             # Make sure the number of steps is positive overall
             # Also use the absolute step size here, as the start and end are defined
             # An mathematically incorrect sign won't matter, as the step size is recalculated
-            self.num_steps = abs(int((self.end_d-self.start_d)/abs(self.step_size)))
+            self.num_steps = abs(int((self.end_d - self.start_d) / abs(self.step_size)))
 
             # Adjust the step size to end up exactly at the end point
-            self.step_size = round((self.end_d-self.start_d)/self.num_steps, 2)
+            self.step_size = round((self.end_d - self.start_d) / self.num_steps, 2)
 
     def to_orca(self):
-        ids_str = ' '.join([str(i-1) for i in self.ids])
+        ids_str = " ".join([str(i - 1) for i in self.ids])
         type = len(self.ids)
 
         if type == 2:
@@ -267,7 +351,7 @@ class Constraint:
             return f"{{ {t} {ids_str} C }}\n"
 
     def to_gaussian(self):
-        ids_str = ' '.join([str(i) for i in self.ids])
+        ids_str = " ".join([str(i) for i in self.ids])
         type = len(self.ids)
 
         if type == 2:
@@ -285,6 +369,21 @@ class Constraint:
         else:
             return f"{t} {ids_str} F\n"
 
+    def to_xtb(self):
+        ids_str = ", ".join(
+            [str(i) for i in self.ids] + ["auto"]
+        )  # Maybe add support for other starting points in the future
+        type = len(self.ids)
+
+        if type == 2:
+            t = "distance"
+        elif type == 3:
+            t = "angle"
+        elif type == 4:
+            t = "dihedral"
+
+        return f"{t}: {ids_str}\n"
+
 
 def parse_freeze_constraints(arr, xyz_str, software=""):
     if len(arr) == 0:
@@ -295,6 +394,7 @@ def parse_freeze_constraints(arr, xyz_str, software=""):
 
     return parse_str_constraints(constr, xyz_str, software=software)
 
+
 def parse_scan_constraints(arr, sfrom, sto, snsteps, sstep, xyz_str, software=""):
     if len(arr) == 0:
         return []
@@ -302,55 +402,84 @@ def parse_scan_constraints(arr, sfrom, sto, snsteps, sstep, xyz_str, software=""
     scans = []
     for ids, fro, to, nsteps, step in zip_longest(arr, sfrom, sto, snsteps, sstep):
         if ids is None:
-            raise InvalidParameter("Not enough sets of atom indices specified for the number of other parameters")
+            raise InvalidParameter(
+                "Not enough sets of atom indices specified for the number of other parameters"
+            )
         _ids = [int(i) for i in ids]
-        scans.append(gen_constraint(_ids, xyz_str, 'scan', start_str=fro, end_str=to, nsteps_str=nsteps, step_str=step, software=software))
+        scans.append(
+            gen_constraint(
+                _ids,
+                xyz_str,
+                "scan",
+                start_str=fro,
+                end_str=to,
+                nsteps_str=nsteps,
+                step_str=step,
+                software=software,
+            )
+        )
 
     return scans
 
+
 def parse_str_constraints(s, xyz_str, software=""):
     if s.strip() == "":
         return []
 
     if software == "":
-        warn("No software specified for the constraints; the behaviour might be incorrect")
+        warn(
+            "No software specified for the constraints; the behaviour might be incorrect"
+        )
 
     constraints = []
-    cs = s.split(';')
+    cs = s.split(";")
     for c in cs:
-        if c.strip() == '':
+        if c.strip() == "":
             continue
 
-        if c.count('/') != 1:
+        if c.count("/") != 1:
             raise InvalidParameter(f"Invalid constraint string: '{c}'")
 
-        specs_str, ids_str = c.lower().split('/')
+        specs_str, ids_str = c.lower().split("/")
 
         try:
-            ids = [int(i) for i in ids_str.split('_')]
+            ids = [int(i) for i in ids_str.split("_")]
         except ValueError:
-            raise InvalidParameter(f"Could not parse the atom numbers from the string '{ids_str}'")
+            raise InvalidParameter(
+                f"Could not parse the atom numbers from the string '{ids_str}'"
+            )
 
-        constraints.append(gen_constraint(ids, xyz_str, *specs_str.split('_'), software=software))
+        constraints.append(
+            gen_constraint(ids, xyz_str, *specs_str.split("_"), software=software)
+        )
 
     return constraints
 
-def gen_constraint(ids, xyz_str, option, start_str=None, end_str=None, nsteps_str=None,
-                   step_str=None, software=""):
-    """
-        Generate a constraint object from arrays of parameters.
-        In case of multiple values in the arrays, each parameter with the same index
-        is assumed to be of the same command constraint.
 
-        The atom indices must already be integers in ids as array of arrays of ints.
+def gen_constraint(
+    ids,
+    xyz_str,
+    option,
+    start_str=None,
+    end_str=None,
+    nsteps_str=None,
+    step_str=None,
+    software="",
+):
+    """
+    Generate a constraint object from arrays of parameters.
+    In case of multiple values in the arrays, each parameter with the same index
+    is assumed to be of the same command constraint.
+
+    The atom indices must already be integers in ids as array of arrays of ints.
 
     """
     t = len(ids)
 
-    if option not in ['scan', 'freeze']:
+    if option not in ["scan", "freeze"]:
         raise InvalidParameter(f"Invalid type of scan: '{specs[0]}'")
 
-    if option == 'scan':
+    if option == "scan":
         if start_str is not None:
             try:
                 start_d = float(start_str)
@@ -398,5 +527,13 @@ def gen_constraint(ids, xyz_str, option, start_str=None, end_str=None, nsteps_st
         num_steps = None
         step_size = None
 
-    return Constraint(scan=option == 'scan', start_d=start_d, end_d=end_d, num_steps=num_steps,
-                      step_size=step_size, ids=ids, xyz=xyz_str, software=software.lower())
+    return Constraint(
+        scan=option == "scan",
+        start_d=start_d,
+        end_d=end_d,
+        num_steps=num_steps,
+        step_size=step_size,
+        ids=ids,
+        xyz=xyz_str,
+        software=software.lower(),
+    )
