@@ -6,16 +6,17 @@ from ccinput.constants import *
 from ccinput.exceptions import *
 
 MEMORY_FACTORS = {
-            'm': 1,
-            'mb': 1,
-            'mib': 1.048576, # (1024/1000)^2
-            'g': 1000,
-            'gb': 1000,
-            'gib': 1073.741824,# 1000*(1024/1000)^3
-            't': 1000000,
-            'tb': 1000000,
-            'tib': 1099511.628,# 1000^2*(1024/1000)^4
-        }
+    "m": 1,
+    "mb": 1,
+    "mib": 1.048576,  # (1024/1000)^2
+    "g": 1000,
+    "gb": 1000,
+    "gib": 1073.741824,  # 1000*(1024/1000)^3
+    "t": 1000000,
+    "tb": 1000000,
+    "tib": 1099511.628,  # 1000^2*(1024/1000)^4
+}
+
 
 def verify_memory_valid(mem):
     if mem < 0:
@@ -26,8 +27,9 @@ def verify_memory_valid(mem):
 
     return round(mem)
 
+
 def standardize_memory(mem):
-    """ Converts a string specifying an amount of memory to an integer number of megabytes """
+    """Converts a string specifying an amount of memory to an integer number of megabytes"""
 
     if isinstance(mem, (int, float)):
         return verify_memory_valid(mem)
@@ -51,35 +53,41 @@ def standardize_memory(mem):
             unit = "mb"
         else:
             if unit not in MEMORY_FACTORS:
-                raise InvalidParameter(f"Unknown unit used for memory specification: '{unit}'")
+                raise InvalidParameter(
+                    f"Unknown unit used for memory specification: '{unit}'"
+                )
 
         try:
             val = float(_val)
         except ValueError:
-            raise InvalidParameter(f"The amount of memory must be a number (received '{mem}')")
-        standard_mem = val*MEMORY_FACTORS[unit]
+            raise InvalidParameter(
+                f"The amount of memory must be a number (received '{mem}')"
+            )
+        standard_mem = val * MEMORY_FACTORS[unit]
         return verify_memory_valid(standard_mem)
     else:
-        raise InvalidParameter(f"Invalid type for the memory specification: {type(mem)}")
+        raise InvalidParameter(
+            f"Invalid type for the memory specification: {type(mem)}"
+        )
 
 
 def standardize_xyz(xyz):
     """
-        Converts variations of the XYZ format into a uniform format for this project.
+    Converts variations of the XYZ format into a uniform format for this project.
 
-        Format:
-            "El1 X1 Y1 Z2\n El2 X2 Y2 Z2 [...]"
+    Format:
+        "El1 X1 Y1 Z2\n El2 X2 Y2 Z2 [...]"
 
-        Example:
-            H 0.0 0.0 0.0
-            H 1.0 0.0 0.0
+    Example:
+        H 0.0 0.0 0.0
+        H 1.0 0.0 0.0
     """
 
     standard_xyz = ""
     if isinstance(xyz, list):
         arr_xyz = xyz
     elif isinstance(xyz, str):
-        arr_xyz = xyz.strip().split('\n')
+        arr_xyz = xyz.strip().split("\n")
     else:
         raise InvalidParameter(f"Cannot parse xyz from type {type(xyz)}")
 
@@ -90,20 +98,24 @@ def standardize_xyz(xyz):
     except ValueError:
         pass
     else:
-        if num_atoms == len(arr_xyz)-2:
+        if num_atoms == len(arr_xyz) - 2:
             arr_xyz = arr_xyz[2:]
         else:
-            raise InvalidXYZ(f"Invalid xyz header: {num_atoms} atoms specified, " +
-                    "but actually contains {len(arr_xyz) - 2} atoms")
+            raise InvalidXYZ(
+                f"Invalid xyz header: {num_atoms} atoms specified, "
+                + f"but actually contains {len(arr_xyz) - 2} atoms"
+            )
 
     for el in arr_xyz:
         line_data = []
 
         if not isinstance(el, str):
-            raise InvalidXYZ(f"Could not parse xyz from array: contains element '{el}' " +
-                    "and not string")
+            raise InvalidXYZ(
+                f"Could not parse xyz from array: contains element '{el}' "
+                + "and not string"
+            )
 
-        if el.strip() == '':
+        if el.strip() == "":
             continue
 
         sel = el.strip().split()
@@ -141,6 +153,7 @@ def standardize_xyz(xyz):
 
     return standard_xyz
 
+
 def parse_xyz_from_file(path):
     if not os.path.isfile(path):
         raise InvalidParameter(f"Input file not found: {path}")
@@ -150,46 +163,61 @@ def parse_xyz_from_file(path):
 
     return standardize_xyz(lines)
 
+
+def indexify(txt):
+    INDEX_CHARS = string.ascii_lowercase + string.digits + "+*"
+    _txt = txt.lower()
+    return "".join([i for i in _txt if i in INDEX_CHARS])
+
+
 def get_theory_level(method):
+    _method = indexify(method)
     for level, methods in THEORY_LEVELS.items():
-        if method.lower() in methods:
+        if _method in methods or _method == level:
             return level
-    return 'dft'
+    return "dft"
+
 
 def get_abs_type(str_type):
     """
-        Converts a string calculation type into the correct CalcType.
-        Takes into account different equivalent ways to write the calculation types.
+    Converts a string calculation type into the correct CalcType.
+    Takes into account different equivalent ways to write the calculation types.
     """
-    _str_type = str_type.lower().strip()
-    if _str_type in STR_TYPES:
-        return STR_TYPES[_str_type]
-    else:
-        raise InvalidParameter(f"Invalid calculation type: '{str_type}'")
+    _str_type = indexify(str_type)
+    for calc_type, synonyms in SYN_TYPES.items():
+        if _str_type in synonyms:
+            return calc_type
+
+    raise InvalidParameter(f"Invalid calculation type: '{str_type}'")
+
 
 def get_abs_software(software):
-    _software = software.lower().strip()
+    _software = indexify(software)
     for s in SYN_SOFTWARE:
         if _software in SYN_SOFTWARE[s] or _software == s:
             return s
     raise InvalidParameter(f"Unknown software package: '{software}'")
 
+
 def get_abs_method(method):
-    _method = method.strip().lower()
+    _method = indexify(method)
     for m in SYN_METHODS:
         if _method in SYN_METHODS[m] or _method == m:
             return m
     raise InvalidParameter(f"Unknown method: '{method}'")
 
+
 def get_abs_basis_set(basis_set):
-    _bs = basis_set.strip().lower().replace('-', '').replace('(', '').replace(')', '').replace(',', '')
+    _bs = indexify(basis_set)
+
     for bs in SYN_BASIS_SETS:
         if _bs.lower() in SYN_BASIS_SETS[bs] or _bs.lower() == bs:
             return bs
     raise InvalidParameter(f"Unknown basis set: '{basis_set}'")
 
+
 def get_abs_solvent(solvent):
-    _solvent = solvent.strip().lower()
+    _solvent = indexify(solvent)
     if _solvent in ["", "vacuum", "vac"]:
         return ""
     for solv in SYN_SOLVENTS:
@@ -197,28 +225,34 @@ def get_abs_solvent(solvent):
             return solv
     raise InvalidParameter(f"Unknown solvent: '{solvent}'")
 
+
 def is_exchange_correlation_combination(method):
     for x in EXCHANGE_FUNCTIONALS:
-        if method[:len(x)] == x:
-            if method[len(x):] in CORRELATION_FUNCTIONALS:
-                return EXCHANGE_FUNCTIONALS[method[:len(x)]] + \
-                       CORRELATION_FUNCTIONALS[method[len(x):]]
+        if method[: len(x)] == x:
+            if method[len(x) :] in CORRELATION_FUNCTIONALS:
+                return (
+                    EXCHANGE_FUNCTIONALS[method[: len(x)]]
+                    + CORRELATION_FUNCTIONALS[method[len(x) :]]
+                )
     return False
 
+
 def get_method(method, software):
     try:
         abs_method = get_abs_method(method)
     except InvalidParameter:
-        if software == 'gaussian':
+        if software == "gaussian":
             # As far as I know, this kind of specification does not apply to ORCA
-            if method.lower()[0] in ['u', 'r']:
+            if method.lower()[0] in ["u", "r"]:
                 try:
                     abs_method = get_abs_method(method[1:])
                 except InvalidParameter:
                     pass
                 else:
                     if abs_method in SOFTWARE_METHODS[software]:
-                        return method[0].upper() + SOFTWARE_METHODS[software][abs_method]
+                        return (
+                            method[0].upper() + SOFTWARE_METHODS[software][abs_method]
+                        )
 
                 xc_check = is_exchange_correlation_combination(method.lower()[1:])
                 if isinstance(xc_check, str):
@@ -236,6 +270,7 @@ def get_method(method, software):
 
         return SOFTWARE_METHODS[software][abs_method]
 
+
 def get_basis_set(basis_set, software):
     try:
         abs_basis_set = get_abs_basis_set(basis_set)
@@ -245,6 +280,7 @@ def get_basis_set(basis_set, software):
 
     return SOFTWARE_BASIS_SETS[software][abs_basis_set]
 
+
 def get_solvent(solvent, software, solvation_model="smd"):
     try:
         abs_solvent = get_abs_solvent(solvent)
@@ -252,7 +288,7 @@ def get_solvent(solvent, software, solvation_model="smd"):
         warn(f"Unknown solvent '{solvent}'")
         return solvent
 
-    if software == "orca" and abs_solvent == "n-octanol":
+    if software == "orca" and abs_solvent == "noctanol":
         # Weird exception in ORCA
         if solvation_model == "smd":
             return "1-octanol"
@@ -262,57 +298,68 @@ def get_solvent(solvent, software, solvation_model="smd"):
 
     return SOFTWARE_SOLVENTS[software][abs_solvent]
 
+
 def has_dispersion_parameters(method, version="d3"):
     try:
         _method = get_abs_method(method)
     except InvalidParameter:
-        warn("Unknown method, could not verify if dispersion parameters are available for it")
-        return True # Don't print a second warning
+        warn(
+            "Unknown method, could not verify if dispersion parameters are available for it"
+        )
+        return True  # Don't print a second warning
 
     if _method in FUNCTIONALS_WITH_DISPERSION_PARAMETERS[version]:
         return True
     return False
 
+
 def clean_xyz(xyz):
-    return ''.join([x if x in string.printable else ' ' for x in xyz])
+    return "".join([x if x in string.printable else " " for x in xyz])
+
 
 def get_distance(xyz, a, b):
-    return np.linalg.norm(xyz[a-1][1] - xyz[b-1][1])
+    return np.linalg.norm(xyz[a - 1][1] - xyz[b - 1][1])
+
 
 def get_angle(xyz, a, b, c):
-    v1 = xyz[a-1][1] - xyz[b-1][1]
-    v2 = xyz[c-1][1] - xyz[b-1][1]
+    v1 = xyz[a - 1][1] - xyz[b - 1][1]
+    v2 = xyz[c - 1][1] - xyz[b - 1][1]
+
+    return (
+        np.arccos(v1.dot(v2) / (np.linalg.norm(v1) * np.linalg.norm(v2))) * 180 / np.pi
+    )
 
-    return np.arccos(v1.dot(v2)/(np.linalg.norm(v1)*np.linalg.norm(v2)))*180/np.pi
 
 def get_dihedral(xyz, a, b, c, d):
-    v1 = xyz[b-1][1] - xyz[a-1][1]
-    v2 = xyz[c-1][1] - xyz[b-1][1]
-    v3 = xyz[d-1][1] - xyz[c-1][1]
+    v1 = xyz[b - 1][1] - xyz[a - 1][1]
+    v2 = xyz[c - 1][1] - xyz[b - 1][1]
+    v3 = xyz[d - 1][1] - xyz[c - 1][1]
 
     n1 = np.cross(v1, v2)
-    n1 = n1/np.linalg.norm(n1)
+    n1 = n1 / np.linalg.norm(n1)
 
     n2 = np.cross(v2, v3)
-    n2 = n2/np.linalg.norm(n2)
+    n2 = n2 / np.linalg.norm(n2)
 
-    m1 = np.cross(n1, v2/np.linalg.norm(v2))
+    m1 = np.cross(n1, v2 / np.linalg.norm(v2))
     x = n1.dot(n2)
     y = m1.dot(n2)
 
-    return np.arctan2(y, x)*180/np.pi
+    return np.arctan2(y, x) * 180 / np.pi
+
 
 def get_npxyz(str_xyz):
-    lines = [i + '\n' for i in clean_xyz(str_xyz).split('\n')]
+    lines = [i + "\n" for i in clean_xyz(str_xyz).split("\n")]
 
     xyz = []
     for line in lines:
-        if line.strip() != '':
+        if line.strip() != "":
             a, x, y, z = line.split()
             xyz.append([a, np.array([float(x), float(y), float(z)])])
 
     return xyz
 
+
 def get_coord(xyz, ids):
     if len(ids) == 2:
         return get_distance(xyz, *ids)
@@ -323,5 +370,6 @@ def get_coord(xyz, ids):
     else:
         raise InvalidParameter(f"Invalid number of atoms: {len(ids)}")
 
+
 def warn(msg):
     print(f"*** {msg} ***")
